import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import React from 'react';
import purebem from 'purebem';
import { formatDatetime } from '@billogram/format';
import { FoundationTether } from '@billogram/foundation-tether';
import isHandheld from '@billogram/is-handheld';
import { pgettext } from '@billogram/lang';
import useAutoFocus from '@billogram/use-auto-focus';
import useClickOutside from '@billogram/use-click-outside';
import useDatepicker from '@billogram/use-datepicker';
import usePrevious from '@billogram/use-previous';
import { DownArrow, End, Enter, Home, LeftArrow, PageDown, PageUp, RightArrow, Space, Tab, UpArrow } from './keys';
import './FoundationDatepicker.scss';

function never() {
  return false;
}

function noop() {}

export default /*#__PURE__*/React.forwardRef(function FoundationDatepicker(_ref, ref) {
  var _ref$autoFocus = _ref.autoFocus,
      autoFocus = _ref$autoFocus === void 0 ? false : _ref$autoFocus,
      _ref$disabled = _ref.disabled,
      disabled = _ref$disabled === void 0 ? false : _ref$disabled,
      _ref$emptyText = _ref.emptyText,
      emptyText = _ref$emptyText === void 0 ? pgettext('foundation-datepicker', 'Label--no-date-yet') : _ref$emptyText,
      _ref$hasGrayedOutPast = _ref.hasGrayedOutPast,
      hasGrayedOutPast = _ref$hasGrayedOutPast === void 0 ? false : _ref$hasGrayedOutPast,
      _ref$onChange = _ref.onChange,
      onChange = _ref$onChange === void 0 ? noop : _ref$onChange,
      _ref$shouldDisableDat = _ref.shouldDisableDate,
      shouldDisableDate = _ref$shouldDisableDat === void 0 ? never : _ref$shouldDisableDat,
      _ref$ui = _ref.ui,
      ui = _ref$ui === void 0 ? 'gray' : _ref$ui,
      _ref$value = _ref.value,
      value = _ref$value === void 0 ? null : _ref$value;

  var _useDatepicker = useDatepicker({
    disabled: disabled,
    onChange: onChange,
    shouldDisableDate: shouldDisableDate,
    value: value
  }),
      change = _useDatepicker.change,
      close = _useDatepicker.close,
      currentHighlightedDate = _useDatepicker.currentHighlightedDate,
      currentMonth = _useDatepicker.currentMonth,
      currentYear = _useDatepicker.currentYear,
      expanded = _useDatepicker.expanded,
      handleKeyDown = _useDatepicker.handleKeyDown,
      moveToNextMonth = _useDatepicker.moveToNextMonth,
      moveToPreviousMonth = _useDatepicker.moveToPreviousMonth,
      prevExpanded = _useDatepicker.prevExpanded,
      toggle = _useDatepicker.toggle,
      weekdays = _useDatepicker.weekdays,
      weeks = _useDatepicker.weeks;

  var datepickerRef = React.useRef(null);
  var calendarRef = React.useRef(null);
  useClickOutside([datepickerRef, calendarRef], close);
  var currentHighlightedDateRef = React.useRef(null);
  var nextMonthButtonRef = React.useRef(null);
  var previousMonthButtonRef = React.useRef(null);
  var buttonRef = React.useRef(null);
  useAutoFocus(buttonRef, autoFocus);
  React.useImperativeHandle(ref, function () {
    return {
      focus: function focus() {
        if (buttonRef.current !== null) {
          buttonRef.current.focus();
        }
      }
    };
  });

  var _React$useState = React.useState('none'),
      _React$useState2 = _slicedToArray(_React$useState, 2),
      focus = _React$useState2[0],
      setFocus = _React$useState2[1];

  React.useLayoutEffect(function () {
    if (prevExpanded) {
      if (expanded) {
        switch (focus) {
          case 'date':
            {
              if (currentHighlightedDateRef.current !== null) {
                currentHighlightedDateRef.current.focus();
              }

              break;
            }

          case 'next':
            {
              if (nextMonthButtonRef.current !== null) {
                nextMonthButtonRef.current.focus();
              }

              break;
            }

          case 'none':
            {
              break;
            }

          case 'previous':
            {
              if (previousMonthButtonRef.current !== null) {
                previousMonthButtonRef.current.focus();
              }

              break;
            }
        }
      } else {
        setFocus('none');

        if (buttonRef.current !== null) {
          buttonRef.current.focus();
        }
      }
    } else {
      if (expanded) {
        setFocus('date');
      }
    }
  }, [prevExpanded, expanded, focus]);
  var prevCurrentHighlightedDate = usePrevious(currentHighlightedDate, value);
  var setCurrentHighlightedDateFocus = React.useCallback(function (el) {
    if (el !== null) {
      currentHighlightedDateRef.current = el;

      if (focus === 'date' && prevCurrentHighlightedDate !== currentHighlightedDate) {
        currentHighlightedDateRef.current.focus();
      }
    }
  }, [currentHighlightedDate, focus, prevCurrentHighlightedDate]);

  function onKeyDownDate(evt) {
    switch (evt.which) {
      case Tab:
        {
          if (previousMonthButtonRef.current !== null && nextMonthButtonRef.current !== null) {
            evt.preventDefault();

            if (evt.shiftKey) {
              setFocus('next');
              nextMonthButtonRef.current.focus();
            } else {
              setFocus('previous');
              previousMonthButtonRef.current.focus();
            }
          }
        }
    }
  }

  function onKeyDownMonthSelector(evt) {
    switch (evt.which) {
      case PageUp:
      case PageDown:
      case End:
      case Home:
      case LeftArrow:
      case UpArrow:
      case RightArrow:
      case DownArrow:
        {
          setFocus('date');
          break;
        }
    }
  }

  function onKeyDownPreviousMonth(evt) {
    switch (evt.which) {
      case Enter:
      case Space:
        {
          evt.preventDefault();
          evt.stopPropagation();
          setFocus('previous');
          moveToPreviousMonth();
          break;
        }

      case Tab:
        {
          if (currentHighlightedDateRef.current !== null && nextMonthButtonRef.current !== null) {
            evt.preventDefault();

            if (evt.shiftKey) {
              setFocus('date');
              currentHighlightedDateRef.current.focus();
            } else {
              setFocus('next');
              nextMonthButtonRef.current.focus();
            }
          }

          break;
        }
    }
  }

  function onKeyDownNextMonth(evt) {
    switch (evt.which) {
      case Enter:
      case Space:
        {
          evt.preventDefault();
          evt.stopPropagation();
          setFocus('next');
          moveToNextMonth();
          break;
        }

      case Tab:
        {
          if (previousMonthButtonRef.current !== null && currentHighlightedDateRef.current !== null) {
            evt.preventDefault();

            if (evt.shiftKey) {
              setFocus('previous');
              previousMonthButtonRef.current.focus();
            } else {
              setFocus('date');
              currentHighlightedDateRef.current.focus();
            }
          }

          break;
        }
    }
  }

  var handleChange = React.useCallback(function (evt) {
    if (evt.target.value === '') {
      change(null);
    } else {
      change(+new Date(evt.target.value));
    }
  }, [change]);
  var setTarget = React.useCallback(function () {
    return datepickerRef.current;
  }, []);
  var block = purebem('foundation-datepicker'); // We intend to follow these resources as closely as possible (not quite there yet though):
  //  - https://www.w3.org/TR/wai-aria-practices/#dialog_modal
  //  - https://www.w3.org/TR/wai-aria-practices/examples/dialog-modal/datepicker-dialog.html

  return /*#__PURE__*/React.createElement("div", {
    className: block(),
    onKeyDown: handleKeyDown,
    ref: datepickerRef
  }, /*#__PURE__*/React.createElement("div", {
    "aria-expanded": expanded ? true : void 0,
    "aria-haspopup": "dialog",
    className: block('button', {
      disabled: disabled,
      expanded: expanded,
      ui: ui
    }),
    onClick: !isHandheld() ? toggle : void 0,
    ref: buttonRef,
    role: "button",
    tabIndex: !disabled ? 0 : void 0
  }, /*#__PURE__*/React.createElement("div", {
    className: block('button-content')
  }, isHandheld() ? /*#__PURE__*/React.createElement("input", {
    className: block('handheld'),
    defaultValue: value !== null ? value : void 0,
    disabled: disabled,
    onChange: handleChange,
    type: "date"
  }) : null, /*#__PURE__*/React.createElement("span", {
    className: block('button-icon')
  }), /*#__PURE__*/React.createElement("span", {
    className: block('button-text')
  }, value !== null ? formatDatetime(value) : emptyText))), !isHandheld() && expanded ? /*#__PURE__*/React.createElement(FoundationTether, {
    target: setTarget
  }, /*#__PURE__*/React.createElement("div", {
    className: block('calendar'),
    ref: calendarRef
  }, /*#__PURE__*/React.createElement("div", {
    className: block('calendar-month-selector'),
    onKeyDown: onKeyDownMonthSelector
  }, /*#__PURE__*/React.createElement("div", {
    "aria-label": "previous year",
    className: block('calendar-month-selector-icon', ['previous']),
    onClick: moveToPreviousMonth,
    onKeyDown: onKeyDownPreviousMonth,
    ref: previousMonthButtonRef,
    role: "button",
    tabIndex: 0
  }), /*#__PURE__*/React.createElement("div", {
    "aria-live": "polite",
    className: block('calendar-month-selector-text')
  }, pgettext('foundation-datepicker', "Month--".concat(currentMonth)), " ", currentYear), /*#__PURE__*/React.createElement("div", {
    "aria-label": "next year",
    className: block('calendar-month-selector-icon', ['next']),
    onClick: moveToNextMonth,
    onKeyDown: onKeyDownNextMonth,
    ref: nextMonthButtonRef,
    role: "button",
    tabIndex: 0
  })), /*#__PURE__*/React.createElement("table", null, /*#__PURE__*/React.createElement("thead", null, /*#__PURE__*/React.createElement("tr", {
    className: block('calendar-week', ['header'])
  }, weekdays.map(function (day) {
    return /*#__PURE__*/React.createElement("th", {
      className: block('calendar-weekday'),
      key: day
    }, pgettext('foundation-datepicker', "WeekdayHeader--".concat(day)));
  }))), /*#__PURE__*/React.createElement("tbody", null, weeks.map(function (days, index) {
    return /*#__PURE__*/React.createElement("tr", {
      className: block('calendar-week'),
      key: index
    }, days.map(function (day) {
      var highlighted = day.isHighlighted,
          outside = day.isOutsideCurrentMonth,
          selected = day.isSelected,
          today = day.isToday;
      var disabled = shouldDisableDate(day.value);
      var grayedOut = day.isPast && hasGrayedOutPast;

      function onClick() {
        change(day.value);
      }

      return /*#__PURE__*/React.createElement("td", {
        className: block('calendar-day'),
        key: day.value
      }, /*#__PURE__*/React.createElement("div", {
        className: block('calendar-day-content', {
          disabled: disabled,
          grayedOut: grayedOut,
          highlighted: highlighted,
          outside: outside,
          selected: selected,
          today: today
        }),
        onClick: !disabled ? onClick : void 0,
        onKeyDown: onKeyDownDate,
        ref: day.isHighlighted ? setCurrentHighlightedDateFocus : void 0,
        role: "button",
        tabIndex: day.isHighlighted ? 0 : void 0
      }, day.date));
    }));
  }))))) : null);
});
//# sourceMappingURL=FoundationDatepicker.js.map